import { formatPriceDiv } from '@/common/format'
import NormalButton from '@/components/normalButton'
import { View, Text } from '@tarojs/components'
import { memo, useState } from 'react'
import styles from './index.module.scss'
import MCheckbox from '@/components/checkbox'

type PropsType = {
  onDelete?: Function
  onSelectCheckbox?: (bool: boolean) => void
  disabled?: boolean
}

export default memo<PropsType>(props => {
  const { onDelete, onSelectCheckbox, disabled = false } = props

  const handleDelete = () => {
    onDelete && onDelete()
  }

  const selectCallBack = () => {
    onSelectCheckbox && onSelectCheckbox(true)
    setSelectAll(true)
  }
  const closeCallBack = () => {
    setSelectAll(false)
    onSelectCheckbox && onSelectCheckbox(false)
  }

  const [isSelectAll, setSelectAll] = useState(false)

  return (
    <View className={styles.bottomBar}>
      <View className={styles.bottomLeft}>
        <MCheckbox
          status={isSelectAll}
          disabled={disabled}
          round
          size='small'
          onSelect={selectCallBack}
          onClose={closeCallBack}
          customStyles={{ color: '$color: #000000, $alpha: 0.6' }}>
          全选
        </MCheckbox>
      </View>
      <View className={styles.bottomRight}>
        <NormalButton round type='danger' size='normal' onClick={handleDelete} customStyles={{ backgroundColor: '#F44761', padding: '0 80rpx' }}>
          <Text style={{ fontSize: '32rpx' }}>删除</Text>
        </NormalButton>
      </View>
    </View>
  )
})
