import { View, Text, Image } from '@tarojs/components'
import MCheckbox from '@/components/checkbox'
import Counter from '@/components/counter'
import { FC, forwardRef, memo } from 'react'
import classnames from 'classnames'
import styles from './index.module.scss'
import { debounce } from '@/common/util'
import { formatImgUrl, formatPriceDiv } from '@/common/format'
import { EnumSaleMode } from '@/common/Enumerate'
import { selectList } from '../../config'
import { Goods, ShoppingDispatchType, ShoppingStateContextValue, useShoppingDispatch, useShoppingState } from '../../context'

type PropsType = {
  state?: Goods
  purchaserId: number
  itemData: Record<string, any> & object
  orderType: EnumSaleMode
}

// 注意：如果在表单控件内部使用 useWatch，由于是直接从Context中取值，memo也会直接失去作用。

let ColorKindItem: FC<PropsType> = props => {
  console.log('Rerender component: ColorKindItem')
  const { state: multipleSelection, purchaserId, itemData, orderType = EnumSaleMode.Bulk } = props
  const dispatch = useShoppingDispatch()
  // console.log('checked==>', checked)

  //格式化金额
  const formatPrice = (price: number) => {
    return Number(formatPriceDiv(price))
  }

  //格式化数量
  const formatCount = itemData => {
    return itemData.sale_mode == EnumSaleMode.Bulk ? itemData.roll : itemData.length / 100
  }

  //格式化单位
  const formatUnit = itemData => {
    return itemData.sale_mode == EnumSaleMode.Bulk ? '条' : '米'
  }

  const handleSelect = () => {
    dispatch({
      type: ShoppingDispatchType.UPDATE_CHANGED_CHECKBOX,
      data: {
        purchaserId: purchaserId,
        multipleSelection: { ...multipleSelection, [itemData.id]: itemData },
      },
    })
  }
  const handleClose = () => {
    const temp = multipleSelection
    delete temp?.[itemData.id]
    dispatch({
      type: ShoppingDispatchType.UPDATE_CHANGED_CHECKBOX,
      data: {
        purchaserId: purchaserId,
        multipleSelection: temp,
      },
    })
  }


  // TODO：需要新增调整条数/米数的接口  并在调整完成后重新请求整个购物车页面
  const getInputValue = debounce(async (num, itemData) => {
    if (itemData.sale_mode === EnumSaleMode.Bulk) {
      itemData.roll = num
    } else {
      itemData.length = num
    }
    
  }, 260)



  return (
    <MCheckbox
      status={multipleSelection?.hasOwnProperty(itemData.id) || false}
      onSelect={handleSelect}
      onClose={handleClose}
      customClassName={classnames(styles.checkbox, multipleSelection?.hasOwnProperty(itemData.id) ? styles.selected : '')}
      customTextClass={styles.colorKindItem}>
      <View className={styles['colorKindItem__left']}>
        <Image className={styles['colorKindItem__left--image']} mode='aspectFill' src={formatImgUrl(itemData.product_color_texture_url)}></Image>
      </View>
      <View className={styles['colorKindItem__center']}>
        <Text className={styles['colorKindItem__center--title']}>
          {itemData.product_code}# {itemData.product_name}
        </Text>
        <View className={styles['colorKindItem__center--detail']}>
          <Text className={styles['colorKindItem__center--ID']}>{itemData.product_color_code}</Text>
          <Text className={styles['colorKindItem__center--description']}>{itemData.product_color_name}</Text>
        </View>
      </View>
      <View className={styles['colorKindItem__right']}>
        <View className={styles['colorKindItem__right--price']}>￥ {formatPrice(itemData.sale_price)}/kg</View>
        <View className={styles['colorKindItem__right--counter']}>
          <Counter
            onBlue={e => getInputValue(e, itemData)}
            defaultNum={formatCount(itemData)}
            step={selectList[orderType].step}
            digits={selectList[orderType].digits}
            onClickBtn={e => getInputValue(e, itemData)}
            unit={formatUnit(itemData)}
            minNum={selectList[orderType].minNum}
            maxNum={selectList[orderType].maxNum}
          />
        </View>
      </View>
    </MCheckbox>
  )
}
// State 分割组件 思路就是把 context直接通过props的形式传给组件，这样的话就解决了context强制刷新memo的问题了
const withStateSlice = (comp, slice) => {
  const MemoComp = memo(comp)
  const Wrapper = (props, ref) => {
    const state = useShoppingState()
    return <MemoComp ref={ref} state={slice(state, props)} {...props} />
  }
  return memo(forwardRef(Wrapper))
}

ColorKindItem = withStateSlice(
  ColorKindItem,
  (state: ShoppingStateContextValue, props: PropsType) => {
    return state.colorStore[props.purchaserId]['multipleSelection']
  },
)

export default ColorKindItem
