import { Dispatch, FC, ReactNode, useEffect, useMemo, useReducer, useRef, useState } from 'react'
import { ShoppingAction, ShoppingCartAction, ShoppingDispatchContext, ShoppingDispatchContextValue, ShoppingDispatchType, shoppingReducer, ShoppingStateContext } from '../../context'
import { GoodsMeta, ColorStore, ShoppingStateContextValue } from '../../context'

export type TriggerCheckboxOptions = {
  colorStore: ColorStore
  currentCheckedPurchaserId: ShoppingStateContextValue['currentCheckedPurchaserId']
  setSelectedAmount: ShoppingDispatchContextValue['UPDATE_SELECTED_AMOUNT']
}

type InitialState = {
  colorStore: ColorStore
  currentCheckedPurchaserId: number
  currentCheckedSaleMode: number
  isManageStatus: boolean
  selectedAmount: number
}

export interface ShoppingCartPropsType {
  initialValues?: ColorStore
  onTriggerCheckbox?: (options: TriggerCheckboxOptions) => void
  children?: ReactNode
}
// 购物车上下文组件
export const ShoppingProvider: FC<ShoppingCartPropsType> = props => {
  const { children, onTriggerCheckbox, initialValues } = props

  const onTriggerCheckboxRef = useRef(onTriggerCheckbox)
  onTriggerCheckboxRef.current = onTriggerCheckbox

  const [state, dispatch] = useReducer<(state: ShoppingStateContextValue, action: ShoppingAction) => InitialState>(shoppingReducer, {
    colorStore: initialValues || {},
    currentCheckedPurchaserId: -1,
    currentCheckedSaleMode: 0,
    isManageStatus: false,
    selectedAmount: 0,
  })

  

  // 这里要在 useEffect 也就是刷新 state 后再调用，否则如果在 onFieldsChangeRef 修改值会覆盖掉上次修改
  useEffect(() => {
    onTriggerCheckboxRef.current?.({
      colorStore: state.colorStore,
      currentCheckedPurchaserId: state.currentCheckedPurchaserId,
      setSelectedAmount: (amount) => dispatch({ type: ShoppingDispatchType.UPDATE_SELECTED_AMOUNT,data: amount }),
    })
  }, [state.colorStore, state.currentCheckedPurchaserId])

  return (
    <ShoppingStateContext.Provider value={state}>
      <ShoppingDispatchContext.Provider value={dispatch}>{children}</ShoppingDispatchContext.Provider>
    </ShoppingStateContext.Provider>
  )
}
