import { Text, View } from '@tarojs/components'
import { FC, forwardRef, memo, useEffect, useMemo, useState, useTransition } from 'react'
import styles from './index.module.scss'
import classnames from 'classnames'
import { formatMeterDiv } from '@/common/format'
import LayoutBlock from '@/components/layoutBlock'
import Tag from '@/components/tag'
import Divider from '@/components/divider'
import ColorKindItem from '../colorKindItem'
import { EnumSaleMode } from '@/common/Enumerate'
import { selectList } from '../../config'
import { Goods, GoodsMeta, ShoppingDispatchType, ShoppingStateContextValue, useShoppingDispatch, useShoppingState } from '../../context'
import IconFont from '@/components/iconfont/iconfont'
import { isEmptyObject } from '@/common/common'
import classNames from 'classnames'
import LoadingCard from '@/components/loadingCard'

interface ButtonPropsType {
  isActive: boolean
  onClick?: Function
  children?: React.ReactNode
  customStyle?: React.CSSProperties
}
// 订单类型
const SaleModeButton: FC<ButtonPropsType> = props => {
  const { onClick, children, isActive = false, customStyle } = props
  const handleClick = () => {
    onClick?.()
  }
  return (
    <View className={classnames(styles.saleModeButton, isActive && styles['saleModeButton--active'])} style={customStyle} onClick={handleClick}>
      <View className={classnames(styles['saleModeButton--text'])}>{children}</View>
    </View>
  )
}

const DrawerButton = memo<{ isOpen: boolean }>(({ isOpen }) => {
  return (
    <View className={styles.drawerButton}>
      {isOpen ? <IconFont name='icon-shouqi' size={52}></IconFont> : <IconFont name='icon-zhankai' size={52}></IconFont>}
    </View>
  )
})

enum BackEndSaleModeListFieldMap {
  bulk_color_list = 0,
  length_cut_color_list = 1,
  weight_cut_color_list = 2,
}

type PropsType = {
  itemData?: ShoppingCartData
  state?: {
    multipleSelection?: GoodsMeta['multipleSelection']
    currentCheckedPurchaserId?: number
  }
}

let ShoppingCartItem: FC<PropsType> = props => {
  const { itemData, state } = props
  const { multipleSelection, currentCheckedPurchaserId } = state!
  const dispatch = useShoppingDispatch()

  const [openDetail, setOpenDetail] = useState(false)

  const handleOpenDetail = () => {
    setOpenDetail(isOpen => !isOpen)
    handleClickLayout()
  }

  const [selected, setSelect] = useState<EnumSaleMode>(0)

  const onSelectOrderType = (type: EnumSaleMode) => {
    if (isPending) return
    setSelect(type)
    dispatch({ type: ShoppingDispatchType.UPDATE_CURRENT_CHECKED_SALEMODE, data: type })
    // 重置预估金额
    dispatch({ type: ShoppingDispatchType.UPDATE_SELECTED_AMOUNT, data: 0 })
  }

  // const { currentCheckedPurchaserId, colorStore } = useShoppingState()

  console.log('Rerender component: shoppingCartItem')

  const handleClickLayout = () => {
    if (currentCheckedPurchaserId === itemData?.purchaser_id) {
      return
    }
    dispatch({ type: ShoppingDispatchType.UPDATE_SELECTED_AMOUNT, data: 0 })
    dispatch({ type: ShoppingDispatchType.UPDATE_CURRENT_CHECKED_SALEMODE, data: selected })
    dispatch({ type: ShoppingDispatchType.UPDATE_CURRENT_CHECKED_PURCHASERID, data: itemData?.purchaser_id as number })
  }

  // 统计已选面料
  const materialChecked = useMemo(() => {
    if (!multipleSelection || isEmptyObject(multipleSelection)) return 0
    return new Set(
      Object.values(multipleSelection)?.reduce((prev, item: Goods) => {
        if (item.sale_mode === selected) {
          return [...prev, item.product_code]
        }
        return prev
      }, []),
    ).size
  }, [multipleSelection, currentCheckedPurchaserId, selected, itemData])

  // 统计已选颜色
  const colorChecked = useMemo(() => {
    if (!multipleSelection || isEmptyObject(multipleSelection)) return 0
    return new Set(
      Object.values(multipleSelection).reduce((prev, item: Goods) => {
        if (item.sale_mode === selected) {
          return [...prev, item.product_color_code]
        }
        return prev
      }, []),
    ).size
  }, [multipleSelection, currentCheckedPurchaserId, selected, itemData])

  // 统计已选条数 / 米数
  const lengthOrRollChecked = useMemo(() => {
    if (!multipleSelection || isEmptyObject(multipleSelection)) return 0
    return (
      Object.values(multipleSelection).reduce((prev, item: Goods) => {
        if (item.sale_mode === selected) {
          return prev + item.count
        }
        return prev
      }, 0) || 0
    )
  }, [multipleSelection, currentCheckedPurchaserId, selected, itemData])

  const [isPending, startTransition] = useTransition()

  return (
    <LayoutBlock
      circle
      customClassName={classnames(styles.layout, itemData?.purchaser_id === currentCheckedPurchaserId ? styles.selected : '')}
      onClick={handleClickLayout}>
      <View className={classNames(styles.checkbox, 'flex-row', 'justify-between')} onClick={handleOpenDetail}>
        <View className={styles.topItem}>
          <View className='flex-row items-center'>
            <View className={styles.topTitle}>{itemData?.purchaser_name}</View>
            <Tag type='info' size='normal' circle customStyle={{ backgroundColor: '#f0f0f0', borderColor: 'transparent' }} plain>
              {itemData?.sale_user_name}
            </Tag>
          </View>
          <View className={styles.summary}>
            <Text>
              已选 {materialChecked} 种面料，{colorChecked} 个颜色，共{' '}
              {selected === EnumSaleMode.Bulk ? `${lengthOrRollChecked} 条` : `${lengthOrRollChecked} 米`}
            </Text>
          </View>
        </View>
        <DrawerButton isOpen={openDetail} />
      </View>
      {/* 减少节点渲染 */}
      {openDetail && (
        <>
          <View className={classnames(styles.detailBox, openDetail ? styles.drawerOpen : styles.drawerClose)}>
            <View className={styles.checkbox}>
              <Divider customClassName={styles.line}></Divider>
            </View>
            <View className={styles.orderType}>
              <View className={styles.orderTitle}>订单类型</View>
              <View className={styles.orderTypeDetail}>
                <SaleModeButton
                  customStyle={{ padding: '0 50rpx' }}
                  isActive={selected === EnumSaleMode.Bulk}
                  onClick={() => onSelectOrderType(EnumSaleMode.Bulk)}>
                  大货
                </SaleModeButton>
                <SaleModeButton
                  customStyle={{ padding: '0 50rpx' }}
                  isActive={selected === EnumSaleMode.Plate}
                  onClick={() => onSelectOrderType(EnumSaleMode.Plate)}>
                  剪板
                </SaleModeButton>
                <SaleModeButton
                  customStyle={{ padding: '0 50rpx' }}
                  isActive={selected === EnumSaleMode.BulkCut}
                  onClick={() => onSelectOrderType(EnumSaleMode.BulkCut)}>
                  散剪
                </SaleModeButton>
              </View>
            </View>
            <View className={styles.orderColor}>
              <Divider customClassName={styles.line}></Divider>
              <View className={classnames(styles.orderTitle, 'justify-between')}>
                <Text>布料颜色</Text>
                <Tag type='danger' size='normal' circle plain>
                  {selectList[selected].title}单位：{selectList[selected].unit}
                </Tag>
              </View>
            </View>
            <View className={styles.orderContainer}>
              <GoodsList itemData={itemData} selected={selected} isPending={isPending} startTransition={startTransition} />
            </View>
          </View>
        </>
      )}
    </LayoutBlock>
  )
}

interface GoodsListPropType {
  itemData?: ShoppingCartData
  selected: EnumSaleMode
  isPending: boolean
  startTransition: React.TransitionStartFunction
}
const GoodsList = memo<GoodsListPropType>(props => {
  console.log('Rerender component: GoodsList')
  const { itemData, selected, isPending, startTransition } = props
  const dispatch = useShoppingDispatch()

  const [component, setComponent] = useState<JSX.Element | null>(null)

  useEffect(() => {
    startTransition(() => {
      setComponent(
        itemData?.[BackEndSaleModeListFieldMap[selected]].length !== 0 ? (
          itemData?.[BackEndSaleModeListFieldMap[selected]].map(item => {
            dispatch({
              type: ShoppingDispatchType.UPDATE_CHANGED_CHECKBOX,
              data: {
                purchaserId: itemData?.purchaser_id!,
                goodsKind: {
                  [item?.id]: {
                    id: item?.id,
                    estimate_amount: item.estimate_amount,
                    product_code: item.product_code,
                    product_color_code: item.product_color_code,
                    sale_mode: item.sale_mode,
                    count: selected === EnumSaleMode.Bulk ? item.roll : Number(formatMeterDiv(item.length)),
                  },
                },
                multipleSelection: {},
              },
            })
            return <ColorKindItem purchaserId={itemData.purchaser_id} key={item.id} itemData={item} orderType={selected}></ColorKindItem>
          })
        ) : (
          <View className={styles.noList}>暂无数据</View>
        ),
      )
    })
  }, [itemData, selected])

  return (
    <>
      {isPending ? (
        <View className='flex-row items-center justify-center' style={{ width: '100%', height: '200px' }}>
          <LoadingCard />
        </View>
      ) : (
        component
      )}
    </>
  )
})
// State 分割组件 思路就是把 context直接通过props的形式传给组件，这样的话就解决了context强制刷新memo的问题了
const withStateSlice = (comp, slice) => {
  const MemoComp = memo(comp, (prevProps, nextProps)=>{
    let needMemo = true
    if (JSON.stringify(prevProps.itemData) !== JSON.stringify(nextProps.itemData)) {
      needMemo = false
    }
    if(JSON.stringify(prevProps.state) !== JSON.stringify(nextProps.state)){
      needMemo = false
    }
    return needMemo
  })
  const Wrapper = (props, ref) => {
    const state = useShoppingState()
    console.log('state===>', state)
    return <MemoComp ref={ref} state={slice(state, props)} {...props} />
  }
  return memo(forwardRef(Wrapper))
}

ShoppingCartItem = withStateSlice(ShoppingCartItem, (state: ShoppingStateContextValue, props) => ({
  multipleSelection: state.colorStore?.[props.itemData?.purchaser_id]?.['multipleSelection'],
  currentCheckedPurchaserId: state.currentCheckedPurchaserId,
}))

export default ShoppingCartItem
